from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from math import *
import sys
import random
import time

initial_width = 40
initial_height = 40
initial_length = 3
initial_dimension = 2

width = 500
height = 500
mwidth = initial_width
mheight = initial_height
length = initial_length
dimension = initial_dimension
walls = []

#http://amertune.blogspot.com/2008/12/maze-generation-in-python.html
def generatemaze():
	global mwidth, mheight, length, walls
	print "-----"
	print "Generating new maze..."

	start = time.clock()
	minwidth = 0
	maxwidth = 0
	minheight = 0
	maxheight = 0

	if mwidth%length == 0:
		minwidth = (-mwidth/length)*length
		maxwidth = (mwidth/length)*length
	else:
		minwidth = ((-mwidth+length)/length)*length
		maxwidth = (mwidth/length)*length

	if mheight%length == 0:
		minheight = (-mheight/length)*length
		maxheight = (mheight/length)*length
	else:
		minheight = ((-mheight+length)/length)*length
		maxheight = (mheight/length)*length

	walls = [(x,y,x+length,y)
		for x in range(minwidth, maxwidth, length)
		for y in range(minheight, maxheight+length, length)]
	walls.extend([(x,y,x,y+length)
		for x in range(minwidth, maxwidth+length, length)
		for y in range(minheight, maxheight, length)])
	cell_sets = [set([(x,y)])
		for x in range(minwidth, maxwidth, length)
		for y in range(minheight, maxheight, length)]

	walls_copy = walls[:]
	random.shuffle(walls_copy)

	for wall in walls_copy:
		if wall[0] != minwidth and wall[1] != minheight and wall[2] != maxwidth and wall[3] != maxheight:
			set_a = None
			set_b = None

			for s in cell_sets:
				if (wall[0], wall[1]) in s:
					set_a = s
				if (wall[2], wall[3]) in s:
					set_b = s

			if set_a is not set_b:
				cell_sets.remove(set_a)
				cell_sets.remove(set_b)
				cell_sets.append(set_a.union(set_b))
				walls.remove(wall)

			drawmaze()

	print "New maze generated."
	print "Maze width:", mwidth
	print "Maze height:", mheight
	print "Cell size:", length
	print "Time taken:", time.clock() - start

def init():
	global width, height
	glClearColor(0.0, 0.0, 0.0, 0.0)
	glClearDepth(1.0) 
	glDepthFunc(GL_LESS)
	glEnable(GL_DEPTH_TEST)
	glShadeModel(GL_SMOOTH)
	glMatrixMode(GL_PROJECTION)
	glLoadIdentity()
	gluPerspective(45.0, float(width)/float(height), 0.1, 100.0)
	glMatrixMode(GL_MODELVIEW)
	print "-----"
	print "Welcome to Maze2D"
	print "This program uses Kruskal's algorithm to generate mazes."
	menu()

def menu():
	print "-----"
	print "(m)enu"
	print "(g)enerate new maze"
	print "Cell (s)ize"
	print "Maze (w)idth"
	print "Maze (h)eight"
	print "Fake (d)imension"
	print "(r)eset"
	print "(esc)ape"

def drawmaze():
	global mwidth, mheight, length, dimension
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
	glLoadIdentity()
	glTranslatef(0, 0, -100)
	glRotatef(0, 0, 0, 0)

	for wall in walls:
		if dimension == 2:
			glBegin(GL_LINES)
			glColor3f(1, 1, 1)
			glVertex2f(wall[0], wall[1])
			glVertex2f(wall[2], wall[3])
			glEnd()
		elif dimension == 3:
			glBegin(GL_QUAD_STRIP)
			glColor3f(1, 1, 1)
			glVertex3f(wall[0], wall[1], 0)
			glVertex3f(wall[2], wall[3], 0)
			glVertex3f(wall[0], wall[1], length)
			glVertex3f(wall[2], wall[3], length)
			glEnd()

	glutSwapBuffers()

def keyboard(key, x, y):
	global mwidth, mheight, length, dimension

	if key == chr(27):
		sys.exit(0)
	elif key == 'g':
		generatemaze()
	elif key == 'm':
		menu()
	elif key == 'w':
		mwidth = mwidth - length
		generatemaze()
	elif key == 'W':
		mwidth = mwidth + length
		generatemaze()
	elif key == 'h':
		mheight = mheight - length
		generatemaze()
	elif key == 'H':
		mheight = mheight + length
		generatemaze()
	elif key == 's':
		if length > 1:
			length = length - 1
			generatemaze()
	elif key == 'S':
		length = length + 1
		generatemaze()
	elif key == 'd':
		if dimension > 2:
			dimension = dimension - 1
			drawmaze()
	elif key == "D":
		if dimension < 3:
			dimension = dimension + 1
			drawmaze()
	elif key == 'r':
		mwidth = initial_width
		mheight = initial_height
		length = initial_length
		dimension = initial_dimension
		generatemaze()

def main():
	glutInit(sys.argv)
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH)
	glutInitWindowSize(width, height)
	glutInitWindowPosition(300, 100)
	glutCreateWindow("Maze2D")
	glutKeyboardFunc(keyboard)
	glutDisplayFunc(drawmaze)
	init()
	glutMainLoop()

if __name__ == '__main__':
    main()